local addon, AS = ...

AS.scannames = {}
AS.auctionVis = false
AS.maxVisible = 1
AS.searchOffset = 0

AS.invisSIZ = 0
AS.invisSCH = 0

local AS_LEFT = 0
local AS_RIGHT = 1

local AS_SCAN_SLF = 1
local AS_SCAN_GRP = 2

local default_settings = { scanmode = AS_SCAN_SLF, side = AS_LEFT, scans={} }

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}				
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function AS.AnchorScanWindow()
	AS.UI.scan:ClearPoint("TOPRIGHT")
	AS.UI.scan:ClearPoint("TOPLEFT")
	AS.UI.scan:ClearPoint("BOTTOMLEFT")
	AS.UI.scan:ClearPoint("BOTTOMRIGHT")
	if AHShortcut_Settings.side == AS_LEFT then
		AS.UI.scan:SetPoint("TOPRIGHT", UI.Native.Auction, "TOPLEFT", -5,0)
		AS.UI.scan:SetPoint("BOTTOMRIGHT", UI.Native.Auction, "BOTTOMLEFT", -5,0)
	else
		AS.UI.scan:SetPoint("TOPLEFT", UI.Native.Auction, "TOPRIGHT", 5,0)
		AS.UI.scan:SetPoint("BOTTOMLEFT", UI.Native.Auction, "BOTTOMRIGHT", 5,0)
	end
end

function AS.Event_Addon_SavedVariables_Load_End(h, a)
	if a == addon.identifier then
		if AHShortcut_Settings == nil then AHShortcut_Settings = {} end
		if shAHShortcut_Settings == nil then shAHShortcut_Settings = { scans={} } end
		MergeTable(AHShortcut_Settings, default_settings)
		AS.AnchorScanWindow()
		AS.UI.searches[1].f:SetVisible(false)
		AS.invisSCH=1
		AS.UpdateSearchList()
		if AHShortcut_Settings.scanmode == AS_SCAN_SLF then
			AS.UI.scanmode:SetTexture(addon.identifier, "img/AH_MODE_SLF.png")
		else
			AS.UI.scanmode:SetTexture(addon.identifier, "img/AH_MODE_GRP.png")
		end
		LibVersionCheck.register(addon.toc.Identifier, addon.toc.Version)
	end
end

local function getRarityRGB(r)
	if r == "uncommon" then			return {r=0.00, g=0.80, b=0.00}
	elseif r == "common" then		return {r=1.00, g=1.00, b=1.00}
	elseif r == "rare" then			return {r=0.15, g=0.51, b=0.99}
	elseif r == "relic" then		return {r=1.00, g=0.60, b=0.00}
	elseif r == "epic" then			return {r=0.69, g=0.28, b=1.00}
	elseif r == "sellable" then		return {r=0.53, g=0.53, b=0.53}
	elseif r == "transcendent" then	return {r=1.00, g=0.00, b=0.00}
	else							return {r=0.00, g=0.00, b=0.00}
	end
end

function AS.UpdateSearchList()
	AS.scannames = {}
	
	local scanlist
	if AHShortcut_Settings.scanmode == AS_SCAN_SLF then
		scanlist = AHShortcut_Settings.scans
	else
		scanlist = shAHShortcut_Settings.scans
	end
	
	for k,v in pairs(scanlist) do
		for sk, sd in pairs(v) do
			table.insert(AS.scannames, {t=k, r=sd.r, c=sd.c})
		end
	end
	table.sort(AS.scannames, function(a,b) return string.lower(a.t) < string.lower(b.t) end)
	
	AS.maxSrch = 0
	
	local ni = #AS.scannames
	local sk
	local ek
	
	if ni <= AS.maxVisible then
		sk=1
		ek=ni
		AS.maxSrch = ni
		AS.UI.sb_bg:SetVisible(false)
	else
		if AS.searchOffset+AS.maxVisible > ni then
			AS.searchOffset = ni-AS.maxVisible
		end
		AS.maxSrch = AS.maxVisible
		
		sk = AS.searchOffset+1
		ek = AS.searchOffset+AS.maxVisible
		
		local bh = AS.UI.sb_bg:GetHeight()
		local sh = (AS.maxVisible/ni) * bh
		local soh = bh-sh
		local so = AS.searchOffset/(ni-AS.maxVisible) * soh
		
		AS.UI.sb:SetHeight(sh)
		AS.UI.sb:SetPoint("TOPLEFT", AS.UI.sb_bg, "TOPLEFT", 0,so)
		AS.UI.sb_bg:SetVisible(true)
	end
	
	for x=sk, ek do
		local k = x-AS.searchOffset
		AS.UI.searches[k].f:SetText(AS.scannames[x].t)
		AS.UI.searches[k].i[1]:SetVisible(AS.scannames[x].c ~= nil)
		AS.UI.searches[k].i[2]:SetVisible(AS.scannames[x].r ~= nil)
		if AS.scannames[x].r then
			local cd = getRarityRGB(AS.scannames[x].r)
			AS.UI.searches[k].i[2]:SetBackgroundColor(cd.r, cd.g, cd.b)
		end
	end
	
	if AS.invisSCH > 0 and AS.invisSCH ~= AS.maxSrch+1 then
		AS.UI.searches[AS.invisSCH].f:SetVisible(true)
	end
	AS.UI.searches[AS.maxSrch+1].f:SetVisible(false)
	AS.invisSCH = AS.maxSrch+1
end

function AS.Event_Auction_Scan(h, t,a)
	local UPDATELIST = false
	if t.type ~= "mine" and t.type ~= "bids" and t.text then
		local scanlist
		if AHShortcut_Settings.scanmode == AS_SCAN_SLF then
			scanlist = AHShortcut_Settings.scans
		else
			scanlist = shAHShortcut_Settings.scans
		end
	
		if scanlist[t.text] == nil then
			scanlist[t.text] = {}
			table.insert(scanlist[t.text], {r=t.rarity, c=t.category})
			UPDATELIST = true
		else 
			local found = false
			for k,v in pairs(scanlist[t.text]) do
				if v.r == t.rarity and v.c == t.category then
					found = true
					break
				end
			end
			if found == false then
				table.insert(scanlist[t.text], {r=t.rarity, c=t.category})
				UPDATELIST = true
			end
		end
	end
	if UPDATELIST then
		AS.UpdateSearchList()
	end
end

function AS.CheckSearchList()
	local trl, trt, trr, trb
	trl, trt, trr, trb = AS.UI.scan:GetTrimDimensions()
	local swH = AS.UI.scan:GetHeight()
	swH = swH - trt - trb
	
	local maxVis = math.floor(swH/(AS.searchHt+2))
	if maxVis > 0 then
		maxVis = maxVis - 2
		AS.maxVisible = maxVis
		AS.invisSIZ = maxVis+1
		AS.UI.sb_bg:SetHeight(maxVis * (AS.searchHt+2))	
		AS.UI.scanmode:SetWidth(trt * 0.75)
		AS.UI.scanmode:SetHeight(trt * 0.75)
		AS.UpdateSearchList()
	end
end

local function is_wheelforward(self, h)
	if AS.searchOffset > 0 then
		AS.searchOffset = AS.searchOffset-1
		AS.UpdateSearchList()
	end
end

local function is_wheelback(self, h)
	if AS.searchOffset+AS.maxVisible < #AS.scannames then
		AS.searchOffset = AS.searchOffset +1
		AS.UpdateSearchList()
	end
end

local function is_leftclick(self, h)
	local d = AS.scannames[self.SEARCHID+AS.searchOffset]
	local sp = { text=d.t, type="search", index=0, sort="pricePerUnit", sortOrder = "descending" }
	local rt = ""
	local ct = ""
	if d.r then
		sp.rarity = d.r
		rt = " [rarity="..d.r.."]"
	end
	if d.c then
		sp.category = d.c
		ct = " [category="..d.c.."]"
	end
	print("Performing AH scan for: "..d.t..rt..ct)
	AS.UI.text:SetText(d.t)
	Command.Auction.Scan(sp)
end

local function is_rightclick(self, h)
	local d = AS.scannames[self.SEARCHID+AS.searchOffset]
	local scanlist
	if AHShortcut_Settings.scanmode == AS_SCAN_SLF then
		scanlist = AHShortcut_Settings.scans
	else
		scanlist = shAHShortcut_Settings.scans
	end

	for k,v in pairs(scanlist[d.t]) do
		if v.r == d.r and v.c == d.c then
			table.remove(scanlist[d.t], k)
			break
		end
	end
	if #scanlist[d.t] == 0 then
		scanlist[d.t] = nil
	end
	AS.UpdateSearchList()
end

local function is_mousein(self, h)
	self:SetBackgroundColor(0,0.75,0)
end

local function is_mouseout(self, h)
	self:SetBackgroundColor(0,0,0)
end


function AS.BuildUI()
	AS.UI = {}
	AS.UI.ctx = UI.CreateContext(addon.identifier)
	AS.UI.ctx:SetVisible(AS.auctionVis)
	
	AS.UI.scan = UI.CreateFrame("RiftWindow", "AS.UI.scan", AS.UI.ctx)
	AS.UI.scan:SetPoint("TOPRIGHT", UI.Native.Auction, "TOPLEFT", -5,0)
	AS.UI.scan:SetPoint("BOTTOMRIGHT", UI.Native.Auction, "BOTTOMLEFT", -5,0)
	AS.UI.scan:SetWidth(192)
	AS.UI.scan:SetTitle("Searches")
	AS.UI.scan:SetLayer(1)
	
	AS.UI.scan:EventAttach(Event.UI.Layout.Size, function()
		AS.CheckSearchList()
	end, "Event.UI.Layout.Size")
	AS.UI.scan:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, is_wheelforward, "Event.UI.Input.Mouse.Wheel.Forward")
	AS.UI.scan:EventAttach(Event.UI.Input.Mouse.Wheel.Back, is_wheelback, "Event.UI.Input.Mouse.Wheel.Back")			

	AS.UI.scanmode = UI.CreateFrame("Texture", "AS.UI.scanmode", AS.UI.scan)
	AS.UI.scanmode:SetPoint("TOPRIGHT", AS.UI.scan, "TOPRIGHT", -6,6)
	
	AS.UI.scanmode:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if AHShortcut_Settings.scanmode == AS_SCAN_SLF then
			AHShortcut_Settings.scanmode = AS_SCAN_GRP
			AS.UI.scanmode:SetTexture(addon.identifier, "img/AH_MODE_GRP.png")
		else
			AHShortcut_Settings.scanmode = AS_SCAN_SLF
			AS.UI.scanmode:SetTexture(addon.identifier, "img/AH_MODE_SLF.png")
		end		
		AS.searchOffset = 0
		AS.UI.searches[AS.maxSrch+1].f:SetVisible(true)
		AS.UpdateSearchList()
	end, "Event.UI.Input.Mouse.Left.Click")
	
	
	AS.UI.searches = {}

	local trl, trt, trr, trb
	trl, trt, trr, trb = AS.UI.scan:GetTrimDimensions()
	
	AS.UI.text = UI.CreateFrame("RiftTextfield", "AS.UI.text", AS.UI.scan)
	AS.UI.text:SetPoint("BOTTOMLEFT", AS.UI.scan, "BOTTOMLEFT", trl, -trb)
	AS.UI.text:SetPoint("BOTTOMRIGHT", AS.UI.scan, "BOTTOMRIGHT", -trr, -trb)
	AS.UI.text:SetBackgroundColor(0.5, 0, 0.5, 0.5)

	AS.UI.sb_bg = UI.CreateFrame("Frame", "AS.UI.sb_bg", AS.UI.scan)
	AS.UI.sb_bg:SetWidth(4)
	AS.UI.sb_bg:SetBackgroundColor(0,0,0)
	AS.UI.sb_bg:SetPoint("TOPLEFT", AS.UI.scan, "TOPLEFT", trl, trt)
	AS.UI.scan:SetLayer(10)
--	AS.UI.sb_bg:SetPoint("BOTTOMLEFT", AS.UI.scan, "BOTTOMLEFT", trr, -trb)
	
	trl = trl+6

	AS.UI.sb = UI.CreateFrame("Frame", "AS.UI.sb", AS.UI.sb_bg)
	AS.UI.sb:SetPoint("TOPLEFT", AS.UI.sb_bg, "TOPLEFT",0,0)
	AS.UI.sb:SetHeight(20)
	AS.UI.sb:SetWidth(4)
	AS.UI.sb:SetBackgroundColor(1,1,1)
	AS.UI.scan:SetLayer(20)
	
	local fctx = AS.UI.scan
	
	for x=1, 100 do
		local f = UI.CreateFrame("Text", "AS.UI.searches.f:"..x, fctx)
		f.SEARCHID = x
		f:SetWidth(192-trl-trr)
		f:SetBackgroundColor(0,0,0)
		f:SetText("Search #"..x)
		f:SetLayer(10)
		if x==1 then
			AS.searchHt = f:GetHeight()
			f:SetPoint("TOPLEFT", AS.UI.scan, "TOPLEFT", trl, trt)		
		else
			f:SetPoint("TOPLEFT", AS.UI.searches[x-1].f, "BOTTOMLEFT", 0,2)		
		end

		f:EventAttach(Event.UI.Input.Mouse.Left.Click, is_leftclick, "Event.UI.Input.Mouse.Left.Click")
		f:EventAttach(Event.UI.Input.Mouse.Right.Click, is_rightclick, "Event.UI.Input.Mouse.Right.Click")
		f:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, is_wheelforward, "Event.UI.Input.Mouse.Wheel.Forward")
		f:EventAttach(Event.UI.Input.Mouse.Wheel.Back, is_wheelback, "Event.UI.Input.Mouse.Wheel.Back")			
		f:EventAttach(Event.UI.Input.Mouse.Cursor.In, is_mousein, "Event.UI.Input.Mouse.Cursor.In")
		f:EventAttach(Event.UI.Input.Mouse.Cursor.Out, is_mouseout, "Event.UI.Input.Mouse.Cursor.Out")

		i = {}
		for ix=1, 4 do
			local xf = UI.CreateFrame("Frame", "AS.UI.searchs.i:"..ix..","..x, f)
			xf:SetLayer(15)
			xf:SetVisible(false)
			xf:SetWidth(20)
			xf:SetHeight(2)
			if ix == 1 then
				xf:SetBackgroundColor(1,1,0)
				xf:SetPoint("TOPLEFT", f, "BOTTOMLEFT",0,0)
			elseif ix == 2 then
				xf:SetBackgroundColor(0,1,0)
				xf:SetPoint("TOPLEFT", i[1], "TOPRIGHT",0,0)
			elseif ix == 3 then
				xf:SetBackgroundColor(0,0,1)
				xf:SetPoint("TOPLEFT", i[2], "TOPRIGHT",0,0)
			else
				xf:SetBackgroundColor(1,0,1)
				xf:SetPoint("TOPLEFT", i[3], "TOPRIGHT",0,0)
			end
			i[ix] = xf
		end
		AS.UI.searches[x] = {f=f, i=i}
		fctx = f
	end
		
	UI.Native.Auction:EventAttach(Event.UI.Native.Loaded, function()
		AS.auctionVis = not AS.auctionVis
		AS.UI.ctx:SetVisible(AS.auctionVis)
		AS.UI.text:SetKeyFocus(false)
	end, "Event.UI.Native.Loaded")
end

function AS.Command_Slash_Register(h,args)
	local r = {}
	for token in string.gmatch(args, "[^%s]+") do
		table.insert(r, token)
	end
	if r[1] == "side" then
		if AHShortcut_Settings.side == AS_LEFT then
			AHShortcut_Settings.side = AS_RIGHT
		else
			AHShortcut_Settings.side = AS_LEFT
		end
		AS.AnchorScanWindow()
	elseif r[1] == "reset" then
		print("Clearing all saved search history")
		if AHShortcut_Settings.scanmode == AS_SCAN_SLF then
			AHShortcut_Settings.scans = {}
		else
			shAHShortcut_Settings.scans = {}
		end
		AS.UpdateSearchList()
	else
		print("Usage:")
		print("/ahshortcut side - Toggles which side of the AH window to attach to")
		print("/ahshortcut reset - Removes all your saved searches")
	end

end

AS.BuildUI()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, AS.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End" )
Command.Event.Attach(Event.Auction.Scan, AS.Event_Auction_Scan, "Event.Auction.Scan")
Command.Event.Attach(Command.Slash.Register("ahshortcut"), AS.Command_Slash_Register, "Command.Slash.Register")

print(string.format("v%s loaded.", addon.toc.Version))